export const hexToHsva = (hex) => rgbaToHsva(hexToRgba(hex));
export const hexToRgba = (hex) => {
    if (hex[0] === '#')
        hex = hex.substr(1);
    if (hex.length < 6) {
        return {
            r: parseInt(hex[0] + hex[0], 16),
            g: parseInt(hex[1] + hex[1], 16),
            b: parseInt(hex[2] + hex[2], 16),
            a: 1
        };
    }
    return {
        r: parseInt(hex.substr(0, 2), 16),
        g: parseInt(hex.substr(2, 2), 16),
        b: parseInt(hex.substr(4, 2), 16),
        a: 1
    };
};
export const hslaStringToHsva = (hslString) => {
    const matcher = /hsla?\((\d+\.?\d*),\s*(\d+\.?\d*)%?,\s*(\d+\.?\d*)%?,?\s*(\d+\.?\d*)?\)/;
    const match = matcher.exec(hslString);
    if (!match)
        return { h: 0, s: 0, v: 0, a: 1 };
    return hslaToHsva({
        h: Number(match[1]),
        s: Number(match[2]),
        l: Number(match[3]),
        a: match[4] !== undefined ? Number(match[4]) : 1
    });
};
export const hslStringToHsva = hslaStringToHsva;
export const hslaToHsva = ({ h, s, l, a }) => {
    s *= (l < 50 ? l : 100 - l) / 100;
    return {
        h: h,
        s: s > 0 ? ((2 * s) / (l + s)) * 100 : 0,
        v: l + s,
        a
    };
};
export const hsvaToHex = (hsva) => rgbaToHex(hsvaToRgba(hsva));
export const hsvaToHsla = ({ h, s, v, a }) => {
    const hh = ((200 - s) * v) / 100;
    return {
        h: h,
        s: hh > 0 && hh < 200 ? ((s * v) / 100 / (hh <= 100 ? hh : 200 - hh)) * 100 : 0,
        l: hh / 2,
        a
    };
};
export const hsvaToHsvString = (hsv) => {
    const { h, s, v } = hsv;
    return `hsv(${h}, ${s}%, ${v}%)`;
};
export const hsvaToHsvaString = (hsva) => {
    const { h, s, v, a } = hsva;
    return `hsva(${h}, ${s}%, ${v}%, ${a})`;
};
export const hsvaToHslString = (hsva) => {
    const { h, s, l } = hsvaToHsla(hsva);
    return `hsl(${h}, ${s}%, ${l}%)`;
};
export const hsvaToHslaString = (hsva) => {
    const { h, s, l, a } = hsvaToHsla(hsva);
    return `hsla(${h}, ${s}%, ${l}%, ${a})`;
};
export const hsvaToRgba = ({ h, s, v, a }) => {
    h = (h / 360) * 6;
    s = s / 100;
    v = v / 100;
    const hh = Math.floor(h), b = v * (1 - s), c = v * (1 - (h - hh) * s), d = v * (1 - (1 - h + hh) * s), module = hh % 6;
    return {
        r: Math.round([v, c, b, b, d, v][module] * 255),
        g: Math.round([d, v, v, c, b, b][module] * 255),
        b: Math.round([b, b, d, v, v, c][module] * 255),
        a
    };
};
export const hsvaToRgbString = (hsva) => {
    const { r, g, b } = hsvaToRgba(hsva);
    return `rgb(${r}, ${g}, ${b})`;
};
export const hsvaToRgbaString = (hsva) => {
    const { r, g, b, a } = hsvaToRgba(hsva);
    return `rgba(${r}, ${g}, ${b}, ${a})`;
};
export const hsvaStringToHsva = (hsvString) => {
    const matcher = /hsva?\((\d+\.?\d*),\s*(\d+\.?\d*)%?,\s*(\d+\.?\d*)%?,?\s*(\d+\.?\d*)?\)/;
    const match = matcher.exec(hsvString);
    if (!match)
        return { h: 0, s: 0, v: 0, a: 1 };
    return {
        h: Number(match[1]),
        s: Number(match[2]),
        v: Number(match[3]),
        a: match[4] !== undefined ? Number(match[4]) : 1
    };
};
export const hsvStringToHsva = hsvaStringToHsva;
export const rgbaStringToHsva = (rgbaString) => {
    const matcher = /rgba?\((\d+),\s*(\d+),\s*(\d+),?\s*(\d+\.?\d*)?\)/;
    const match = matcher.exec(rgbaString);
    if (!match)
        return { h: 0, s: 0, v: 0, a: 1 };
    return rgbaToHsva({
        r: Number(match[1]),
        g: Number(match[2]),
        b: Number(match[3]),
        a: match[4] !== undefined ? Number(match[4]) : 1
    });
};
export const rgbStringToHsva = rgbaStringToHsva;
const format = (number) => {
    const hex = number.toString(16);
    return hex.length < 2 ? '0' + hex : hex;
};
export const rgbaToHex = ({ r, g, b }) => {
    return '#' + format(r) + format(g) + format(b);
};
export const rgbaToHsva = ({ r, g, b, a }) => {
    const max = Math.max(r, g, b);
    const delta = max - Math.min(r, g, b);
    // prettier-ignore
    const hh = delta
        ? max === r
            ? (g - b) / delta
            : max === g
                ? 2 + (b - r) / delta
                : 4 + (r - g) / delta
        : 0;
    return {
        h: Math.round(60 * (hh < 0 ? hh + 6 : hh)),
        s: Math.round(max ? (delta / max) * 100 : 0),
        v: Math.round((max / 255) * 100),
        a
    };
};
export const rgbaToRgb = ({ r, g, b }) => ({ r, g, b });
export const hslaToHsl = ({ h, s, l }) => ({ h, s, l });
export const hsvaToHsv = ({ h, s, v }) => ({ h, s, v });
//# sourceMappingURL=convert.js.map